@@ -1,9 +1,9 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2006 Kry ( elkry@users.sourceforge.net / http://www.amule.org )
-// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
+// Copyright (c) 2003-2011 Angel Vidal ( kry@amule.org )
+// Copyright (c) 2003-2011 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2002-2011 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -27,56 +27,35 @@
 #ifndef WEBSERVER_H
 #define WEBSERVER_H
 
-#ifdef __WXMSW__
-	#include <wx/msw/winundef.h>
-#endif
-
-#include <wx/wx.h>
-
-#ifdef __WXMSW__
-	#include <winsock.h>
-#else
-	#include <sys/types.h>
-	#include <sys/socket.h>
-	#include <netinet/in.h>
-	#include <arpa/inet.h>
-#endif
-#include <zlib.h>		// Needed for Bytef etc.
-
 #ifdef WITH_LIBPNG
 	#include <png.h>
 #endif
 
-#include <list>
-#include <map>
-#include <vector>
-
-#include <wx/filename.h>
-
-#include <ec/ECPacket.h>
-
-#include "Types.h"
-#include "OtherFunctions.h"
 #include "WebInterface.h"
 #include "KnownFile.h"
 #include "RLE.h"
 #include "OtherStructs.h"
+#include <ec/cpp/ECID.h>	// Needed for CECID
 
-#include "kademlia/utils/UInt128.h" // Need for UInt128
+#ifdef ENABLE_UPNP
+#	include "UPnPBase.h"
+#endif
+
+#include <wx/datetime.h>  // For DownloadFile::wxtLastSeenComplete
+
+#ifdef _MSC_VER
+#define strncasecmp _strnicmp
+#define snprintf sprintf_s
+#define atoll _atoi64
+#define strdup _strdup
+#endif
 
-//class TransferredData;
-class CWSThread;
 class CWebSocket;
 class CMD4Hash;
 
 #define SESSION_TIMEOUT_SECS	300	// 5 minutes session expiration
 #define SHORT_FILENAME_LENGTH	40	// Max size of file name.
 
-//
-//uint8 GetHigherPrio(uint32 prio, bool autoprio);
-//uint8 GetHigherPrioShared(uint32 prio, bool autoprio);
-//uint8 GetLowerPrio(uint32 prio, bool autoprio);
-//uint8 GetLowerPrioShared(uint32 prio, bool autoprio);
 wxString _SpecialChars(wxString str);
 
 class CEC_PartFile_Tag;
@@ -86,7 +65,13 @@ class CEC_SearchFile_Tag;
 class CProgressImage;
 class CEC_KadNode_Tag;
 
-class DownloadFile {
+class CURLDecoder
+{
+      public:
+	static wxString	Decode(const wxString& url);
+};
+
+class DownloadFile : public CECID {
 	public:
 		wxString	sFileName;
 		uint8		nFileStatus;
@@ -104,21 +89,24 @@ class DownloadFile {
 		wxString	sFileHash;
 		wxString	sED2kLink;
 		uint8		nCat;
+		wxDateTime	wxtLastSeenComplete;
 
 		CMD4Hash	nHash;
 		
 		CProgressImage *m_Image;
 		PartFileEncoderData m_Encoder;
+		ArrayOfUInts16 m_PartInfo;
 		std::vector<Gap_Struct> m_ReqParts;
+		ArrayOfUInts64	m_Gaps;
 
 		// container require this		
 		static class DownloadFileInfo *GetContainerInstance();
 		DownloadFile(CEC_PartFile_Tag *);
 		void ProcessUpdate(CEC_PartFile_Tag *);
-		CMD4Hash ID() { return nHash; }
+		uint32 ID() { return ECID(); }
 };
 
-class SharedFile {
+class SharedFile : public CECID {
 	public:
 		wxString	sFileName;
 		uint64		lFileSize;
@@ -138,7 +126,7 @@ class SharedFile {
 		static class SharedFileInfo *GetContainerInstance();
 		SharedFile(CEC_SharedFile_Tag *);
 		void ProcessUpdate(CEC_SharedFile_Tag *);
-		CMD4Hash ID() { return nHash; }
+		uint32 ID() { return ECID(); }
 };
 
 class ServerEntry {
@@ -156,26 +144,25 @@ class ServerEntry {
 		uint32 ID() { return nServerIP; }
 };
 
-class UploadFile {
+// This is a client we are uploading to, not a file
+class UploadFile : public CECID {
 	public:
 		wxString  sUserName;
 		uint32 nTransferredUp;
 		uint32 nTransferredDown;
 		uint32 nSpeed;
-		//
-		// Don't need filename - sharedfiles already have it
-		CMD4Hash  nHash;
+		uint32 nUploadFile;		// ECID of shared file uploading to client
 
 		UploadFile(CEC_UpDownClient_Tag *tag);
 		
 		static class UploadsInfo *GetContainerInstance();
-		CMD4Hash ID() { return nHash; }
+		uint32 ID() { return ECID(); }
 };
 
-class SearchFile {
+class SearchFile : public CECID {
 	public:
 		wxString sFileName;
-		unsigned long lFileSize;
+		uint64 lFileSize;
 		CMD4Hash  nHash;
 		wxString  sHash;
 		long lSourceCount;
@@ -185,7 +172,7 @@ class SearchFile {
 		
 		void ProcessUpdate(CEC_SearchFile_Tag *);
 		static class SearchInfo *GetContainerInstance();
-		CMD4Hash ID() { return nHash; }
+		uint32 ID() { return ECID(); }
 };
 
 
@@ -269,6 +256,19 @@ class UpdatableItemsContainer : public ItemsContainer<T> {
 			// avoid creating nodes
 			return m_items_hash.count(id) ? m_items_hash[id] : NULL;
 		}
+
+		T * GetByHash(const CMD4Hash &fileHash)
+		{
+			T * ret = 0;
+			for (typename std::map<I, T *>::iterator it = m_items_hash.begin(); it != m_items_hash.end(); it++) {
+				if (it->second->nHash == fileHash) {
+					ret = it->second;
+					break;
+				}
+			}
+			return ret;
+		}
+
 		/*!
 		 * Process answer of update request, create list of new items for
 		 * full request later. Also remove items that no longer exist in core
@@ -276,8 +276,8 @@ class UpdatableItemsContainer : public ItemsContainer<T> {
 		void ProcessUpdate(const CECPacket *reply, CECPacket *full_req, int req_type)
 		{
 			std::set<I> core_files;
-			for (int i = 0;i < reply->GetTagCount();i++) {
-				G *tag = (G *)reply->GetTagByIndex(i);
+			for (CECPacket::const_iterator it = reply->begin(); it != reply->end(); it++) {
+				G *tag = (G *) & *it;
 		
 				core_files.insert(tag->ID());
 				if ( m_items_hash.count(tag->ID()) ) {
@@ -312,8 +312,8 @@ class UpdatableItemsContainer : public ItemsContainer<T> {
 		
 		void ProcessFull(const CECPacket *reply)
 		{
-			for (int i = 0;i < reply->GetTagCount();i++) {
-				G *tag = (G *)reply->GetTagByIndex(i);
+			for (CECPacket::const_iterator it = reply->begin(); it != reply->end(); it++) {
+				G *tag = (G *) & *it;
 				// initialize item data from EC tag
 				T item(tag);
 				T *real_ptr = AddItem(item);
@@ -342,7 +342,7 @@ class UpdatableItemsContainer : public ItemsContainer<T> {
 			delete reply;
 		
 			// Phase 3: request full info about files we don't have yet
-			if ( req_full.GetTagCount() ) {
+			if ( req_full.HasChildTags() ) {
 				reply = this->m_webApp->SendRecvMsg_v2(&req_full);
 				if ( !reply ) {
 					return false;
@@ -379,7 +379,7 @@ class ServersInfo : public ItemsContainer<ServerEntry> {
 };
 
 
-class SharedFileInfo : public UpdatableItemsContainer<SharedFile, CEC_SharedFile_Tag, CMD4Hash> {
+class SharedFileInfo : public UpdatableItemsContainer<SharedFile, CEC_SharedFile_Tag, uint32> {
 	public:
 		// can be only one instance.
 		static SharedFileInfo *m_This;
@@ -390,7 +390,7 @@ class SharedFileInfo : public UpdatableItemsContainer<SharedFile, CEC_SharedFile
 
 };
 
-class SearchInfo : public UpdatableItemsContainer<SearchFile, CEC_SearchFile_Tag, CMD4Hash> {
+class SearchInfo : public UpdatableItemsContainer<SearchFile, CEC_SearchFile_Tag, uint32> {
 	public:
 		static SearchInfo *m_This;
 		
@@ -402,7 +402,7 @@ class SearchInfo : public UpdatableItemsContainer<SearchFile, CEC_SearchFile_Tag
 
 
 class CImageLib;
-class DownloadFileInfo : public UpdatableItemsContainer<DownloadFile, CEC_PartFile_Tag, CMD4Hash> {
+class DownloadFileInfo : public UpdatableItemsContainer<DownloadFile, CEC_PartFile_Tag, uint32> {
 		CImageLib *m_ImageLib;
 		
 		// parameters of progress images
@@ -472,16 +472,6 @@ class CProgressImage : public virtual CAnyImage {
 		
 		wxString m_template;
 		
-		//
-		// sorted list of gaps
-		int m_gap_buf_size, m_gap_alloc_size;
-		Gap_Struct *m_gap_buf;
-		
-		void ReallocGapBuffer();
-		void InitSortedGaps();
-		// for qsort
-		static int compare_gaps(const void *, const void *);
-		
 		//
 		// Turn list of gaps, partstatus into array of color strips
 		typedef struct Color_Gap_Struct : public Gap_Struct {
@@ -649,31 +639,18 @@ class CDynStatisticImage : public virtual CDynPngImage {
 #endif
 
 class CImageLib {
-		std::map<wxString, CAnyImage *> m_image_map;
+		typedef std::map<wxString, CAnyImage *> ImageMap;
+		ImageMap m_image_map;
 		wxString m_image_dir;
 	public:
 		CImageLib(wxString image_dir);
 		~CImageLib();
 		
-		CAnyImage *GetImage(wxString &name);
+		CAnyImage *GetImage(const wxString &name);
 		void AddImage(CAnyImage *img, const wxString &name);
 		void RemoveImage(const wxString &name);
 };
 
-class CUrlDecodeTable {
-		static CUrlDecodeTable*		ms_instance;
-		static wxCriticalSection	ms_instance_guard;
-
-		wxString m_enc_u_str[256], m_enc_l_str[256], m_dec_str[256];
-
-		CUrlDecodeTable();
-		
-	public:
-		static CUrlDecodeTable* GetInstance();
-
-		void DecodeString(wxString &str);
-};
-
 class CParsedUrl {
 		wxString m_path, m_file;
 		std::map<wxString, wxString> m_params;
@@ -699,14 +676,21 @@ struct ThreadData {
 	CWebSocket	*pSocket;
 };
 
-/*
- * In transition period I want both versions of amuleweb available: CPP and PHP
- */
-class CWebServerBase {
-	protected:
-		CWSThread *wsThread;
-		wxMutex *m_mutexChildren;
+enum {
+    // Socket handlers
+    ID_WEBLISTENSOCKET_EVENT = wxID_HIGHEST+123,  // random safe ID
+    ID_WEBCLIENTSOCKET_EVENT,
+};
 
+#ifdef ENABLE_UPNP
+class CUPnPControlPoint;
+class CUPnPPortMapping;
+#endif
+
+class CWebServerBase : public wxEvtHandler {
+	protected:
+		wxSocketServer *m_webserver_socket;
+		
 		ServersInfo m_ServersInfo;
 		SharedFileInfo m_SharedFileInfo;
 		DownloadFileInfo m_DownloadFileInfo;
@@ -723,18 +707,27 @@ class CWebServerBase {
 		int GzipCompress(Bytef *dest, uLongf *destLen,
 			const Bytef *source, uLong sourceLen, int level);
 			
-	friend class CWebSocket;
-	friend class CWSThread;		// to access the wsThread member
-	friend class CPhPLibContext;
+		friend class CWebSocket;
+		friend class CPhPLibContext;
 	
+		bool m_upnpEnabled;
+		int m_upnpTCPPort;
+#ifdef ENABLE_UPNP
+		CUPnPControlPoint *m_upnp;
+		std::vector<CUPnPPortMapping> m_upnpMappings;
+#endif
+
+		void OnWebSocketServerEvent(wxSocketEvent& event);
+		void OnWebSocketEvent(wxSocketEvent& event);
+		DECLARE_EVENT_TABLE();
 	public:
 		CWebServerBase(CamulewebApp *webApp, const wxString& templateDir);
 		virtual ~CWebServerBase() { }
 
 		void Send_Discard_V2_Request(CECPacket *request);
 
-		virtual void StartServer() = 0;
-		virtual void StopServer() = 0;
+		void StartServer();
+		void StopServer();
 
 		void Print(const wxString &s);
 
@@ -799,8 +792,6 @@ class CScriptWebServer : public CWebServerBase {
 		CScriptWebServer(CamulewebApp *webApp, const wxString& templateDir);
 		~CScriptWebServer();
 
-		virtual void StartServer();
-		virtual void StopServer();
 };
 
 class CNoTemplateWebServer : public CScriptWebServer {
@@ -812,3 +803,4 @@ class CNoTemplateWebServer : public CScriptWebServer {
 };
 
 #endif // WEBSERVER_H
+// File_checked_for_headers
